@@ -74,7 +74,17 @@ sub _rd2ymd
 
     if ( $_[0] )
     {
-        my $dow = (( $rd + 6 ) % 7 ) + 1;
+        my $dow;
+
+        if ( $rd < -6 )
+        {
+            $dow = ( $rd + 6 ) % 7;
+            $dow += $dow ? 8 : 1;
+        }
+        else
+        {
+            $dow = ( ( $rd + 6 ) % 7 ) + 1;
+        }
 
         my $doy =
             $class->_end_of_last_month_day_of_year( $y, $m );
@@ -143,9 +153,12 @@ sub _seconds_as_components
     shift;
     my $secs = shift;
     my $utc_secs = shift;
+    my $modifier = shift || 0;
 
     use integer;
 
+    $secs -= $modifier;
+
     my $hour = $secs / 3600;
     $secs -= $hour * 3600;
 
@@ -206,7 +219,7 @@ sub _is_leap_year
 
 sub _day_length { DateTime::LeapSecond::day_length($_[1]) }
 
-sub _leap_seconds { DateTime::LeapSecond::leap_seconds($_[1]) }
+sub _accumulated_leap_seconds { DateTime::LeapSecond::leap_seconds($_[1]) }
 
 
 1;
